`timescale 1ns / 1ps

/*
 * add the following to your own
 * ML505Top module, in the appropriate places
 */
module ML505Top(
    SRAM_ADV_LD_B,
    SRAM_BW,
    SRAM_CLK,
    SRAM_CS_B,
    SRAM_D,
    SRAM_DQP,
    SRAM_A,
    SRAM_WE_B,
    SRAM_MODE,
    SRAM_OE_B,

    DVI_D,
    DVI_DE,
    DVI_H,
    DVI_RESET_B,
    DVI_V,
    DVI_XCLK_N,
    DVI_XCLK_P,

    IIC_SCL_VIDEO,
    IIC_SDA_VIDEO
);

output                          SRAM_ADV_LD_B;
output      [3:0]               SRAM_BW;
output                          SRAM_CLK;
output                          SRAM_CS_B;
inout       [31:0]              SRAM_D;
inout       [3:0]               SRAM_DQP;
output      [17:0]              SRAM_A;
output                          SRAM_WE_B;
output                          SRAM_MODE;
output                          SRAM_OE_B;

output      [11:0]              DVI_D;
output                          DVI_DE;
output                          DVI_H;
output                          DVI_RESET_B;
output                          DVI_V;
output                          DVI_XCLK_N;
output                          DVI_XCLK_P;

inout                           IIC_SCL_VIDEO;
inout                           IIC_SDA_VIDEO;

wire                            cpureset;
wire                            dvireset;

wire                            clock125mhz;
wire                            clock50mhz;

wire        [17:0]              address_from_video;
wire                            address_from_video_valid;

wire                            video_ready, video_valid;
wire        [23:0]              video_bits;

SourceOfClocks clocksource(
    .Clock100MHz(               USER_CLK),
    .Clock125MHz(               clock125mhz),
    .Clock50MHz(                clock50mhz)
);

ResetControl resetcontrol(
    .Clock125(                  clock125mhz),
    .Clock50(                   clock50mhz),

    .CPUTrigger(                GPIO_SW_C),
    .DVITrigger(                GPIO_SW_C),

    .CPUReset(                  cpureset),
    .DVIReset(                  dvireset),
);

assign SRAM_ADV_LD_B        = 1'b0;
assign SRAM_CLK             = clock125mhz;
assign SRAM_CS_B            = 1'b0;
assign SRAM_DQP             = 4'b0;
assign SRAM_MODE            = 1'b0;
assign SRAM_OE_B            = 1'b0;

Video video(
    .Reset(                     cpureset),

    .FrameBufferClkIn(          clock125mhz),
    .VideoClkIn(                clock50mhz),

    .PixelsValid(               ),
    .Pixel0In(                  ),
    .Pixel1In(                  ),

    .Address(                   address_from_video),
    .AddressValid(              address_from_video_valid),

    .VideoOut(                  video_bits),
    .VideoReady(                video_ready),
    .VideoValid(                video_valid)
);

/*
 * look in DVI module for explanation
 * of parameters
 */
DVI #(
    .ClockFreq(                 50000000),
    .Width(                     1040),
    .FrontH(                    56),
    .PulseH(                    120),
    .BackH(                     64),
    .Height(                    666),
    .FrontV(                    37),
    .PulseV(                    6),
    .BackV(                     23)
) dvi(
    .Clock(                     clock50mhz),
    .Reset(                     dvireset),
    .DVI_D(                     DVI_D),
    .DVI_DE(                    DVI_DE),
    .DVI_H(                     DVI_H),
    .DVI_V(                     DVI_V),
    .DVI_RESET_B(               DVI_RESET_B),
    .DVI_XCLK_N(                DVI_XCLK_N),
    .DVI_XCLK_P(                DVI_XCLK_P),
    .I2C_SCL_DVI(               IIC_SCL_VIDEO),
    .I2C_SDA_DVI(               IIC_SDA_VIDEO),
    .Video(                     video_bits),
    .VideoReady(                video_ready),
    .VideoValid(                video_valid)
);

endmodule
