/*
 * this program should display horizontal
 * 
 * red
 * green
 * blue
 *
 * then a black screen, then vertical
 *
 * red green blue
 *
 */
typedef unsigned int uint32_t;
typedef int int32_t;

/*
 * sets a pixel in the frame buffer at (x, y) to c
 */
inline static void setpixel(uint32_t x, uint32_t y, uint32_t c)
{
    // follows our frame buffer address convention
    // 32b1000_0000_00yy_yyyy_yyyy_xxxx_xxxx_xx00
    uint32_t address = 0x80000000 | ((y & 0x3FF) << 12) | ((x & 0x3FF) <<2);
    volatile uint32_t* p = (volatile uint32_t*)address;

    // be careful not to do
    // *p = c & 0xFFFF
    // gcc will infer a sh(store halfword) instruction, which we don't have
    *p = c;
}

/*
 * be careful about your compiler optimization level
 * gcc may optimize this out, stick to -O0 to be safe
 */
#define PAUSE 0x1000000
static void sleep()
{
    for (uint32_t i = 0; i < PAUSE; i++) {
        /* sleep */
    }
}

int32_t main(void)
{
    for ( ; ; ) {
        // red vertical stripe
        for (uint32_t j = 0; j < 266; j++) {
            for (uint32_t i = 0; i < 600; i++) {
                setpixel(j, i, 0xF100);
            }
        }
        // green vertical stripe
        for (uint32_t j = 266; j < 266*2; j++) {
            for (uint32_t i = 0; i < 600; i++) {
                setpixel(j, i, 0x07E0);
            }
        }
        // blue vertical stripe
        for (uint32_t j = 266*2; j < 800; j++) {
            for (uint32_t i = 0; i < 600; i++) {
                setpixel(j, i, 0x001F);
            }
        }

	    sleep();
        // blank screen
        for (uint32_t j = 0; j < 600; j++) {
            for (uint32_t i = 0; i < 800; i++) {
                setpixel(i, j, 0x0000);
            }
        }
	    sleep();

        // red horizontal stripe
        for (uint32_t j = 0; j < 200; j++) {
            for (uint32_t i = 0; i < 800; i++) {
                setpixel(i, j, 0xF100);
            }
        }
        // green horizontal stripe
        for (uint32_t j = 200; j < 400; j++) {
            for (uint32_t i = 0; i < 800; i++) {
                setpixel(i, j, 0x07E0);
            }
        }
        // blue horizontal stripe
        for (uint32_t j = 400; j < 600; j++) {
            for (uint32_t i = 0; i < 800; i++) {
                setpixel(i, j, 0x001F);
            }
        }

	    sleep();
        // blank screen
        for (uint32_t j = 0; j < 600; j++) {
            for (uint32_t i = 0; i < 800; i++) {
                setpixel(i, j, 0x0000);
            }
        }
	    sleep();
    }
    return 0;
}
